#ifndef _EX_Device_H
#define _EX_Device_H

#include "Prefix.h"
#include "EARTH_PT3.h"
#include "EX_Stream.h"
#include "EX_Utility.h"
#include "OS_Thread.h"
#include "OS_Time.h"

#if 2 <= PUBLIC_LEVEL
#include "EARTH_PT3_.h"
#include "EX_Device_.h"
#endif

#include "EX_Status.h"
#include "EX_Test.h"
#include "EX_Errata.h"

namespace EARTH {
namespace EX {
	class Device {
	public:
		Device()
		{
			mDevice = NULL;

			#if 2 <= PUBLIC_LEVEL
			mDevice_ = NULL;
			#endif

			mStatus = NULL;
			mTest = NULL;
			mErrata = NULL;

			for (uint32 i=0; i<sizeof(mStream)/sizeof(*mStream); i++) {
				mStream[i] = NULL;
			}
			mStreamBusy = false;

			mIsOpen = false;
			mIsValid = false;
		}

		~Device()
		{
			delete_(mStatus);
			delete_(mTest);
			delete_(mErrata);

			#if 2 <= PUBLIC_LEVEL
			delete_(mDevice_);
			#endif

			for (uint32 i=0; i<sizeof(mStream)/sizeof(*mStream); i++) {
				delete_(mStream[i]);
			}

			if (mDevice) {
				status status;

				if (mIsOpen) {
					status = mDevice->Close();
					if (U::Error(L"Device::Close()", status)) return;
					mIsOpen = false;
				}

				status = mDevice->Delete();
				if (U::Error(L"Device::Delete()", status)) return;

				mDevice = NULL;
			}
		}

		void Run(PT::Bus *bus, const PT::Bus::DeviceInfo *deviceInfo)
		{
			Init(bus, deviceInfo);
			if (mIsValid == false) return;

			#if PUBLIC_LEVEL == 2
			mDevice_->Update();
			return;
			#endif

			Menu1();
		}

	protected:
		PT::Device		*mDevice;

		#if 2 <= PUBLIC_LEVEL
		EX::Device_		*mDevice_;
		#endif

		EX::Test	*mTest;
		EX::Errata	*mErrata;
		Status		*mStatus;
		Stream		*mStream[4];
		bool		mIsOpen;
		bool		mIsValid;
		bool		mStreamBusy;

		void Init(PT::Bus *bus, const PT::Bus::DeviceInfo *deviceInfo)
		{
			status	status;

			#if 2 <= PUBLIC_LEVEL
			PT::Device_ *device_;
			#endif

			status = bus->NewDevice(deviceInfo, &mDevice,
				#if 2 <= PUBLIC_LEVEL
				&device_
				#else
				NULL
				#endif
			);
			if (U::Error(L"Bus::NewDevice()", status)) return;

			#if 2 <= PUBLIC_LEVEL
			mDevice_ = new Device_(mDevice, device_, deviceInfo->PTn);
			#endif

			#if PUBLIC_LEVEL == 2
			uint8	minFpgaVersion = 0x01,
					maxFpgaVersion = 0x03;
			mDevice_->SetUpdateMode(minFpgaVersion, maxFpgaVersion);
			#endif

			_ASSERT(mStatus == NULL);
			mStatus = new Status(mDevice);

			_ASSERT(mTest == NULL);
			mTest = new Test(mDevice);

			_ASSERT(mErrata == NULL);
			mErrata = new Errata(mDevice);

			_ASSERT(mDevice);
			status = mDevice->Open();

			if (status == PT::STATUS_FPGA_VERSION_ERROR) {
				PT::Device::ConstantInfo	info;
				status = mDevice->GetConstantInfo(&info);
				uint8	version = info.Version_FPGA;

				#if PUBLIC_LEVEL != 2
				::wprintf(L"FPGA Hԍ 0x%02x ɂ͑ΉĂ܂B", version);
				if (version <= 0x03) {
					::wprintf(L"HXVĂB");
				}
				::wprintf(L"\n");
				#else
				::wprintf(L"FPGA Ho[W 0x%02x łB̃c[̍XVΏۂ 0x%02x`0x%02x łB\n",
					version, minFpgaVersion, maxFpgaVersion);
				#endif
				return;
			}

			if (U::Error(L"Device::Open()", status)) {
				return;
			}
			mIsOpen = true;

			#if PUBLIC_LEVEL == 2
			mIsValid = true;
			return;
			#endif

			status = mDevice->InitTuner();
			if (U::Error(L"Device::InitTuner()", status)) return;

			for (uint32 i=0; i<2; i++) {
				for (uint32 j=0; j<2; j++) {
					status = mDevice->SetTunerSleep(static_cast<PT::Device::ISDB>(j), i, false);
					if (U::Error(L"Device::SetTunerSleep()", status)) return;
				}
			}

#if 0
			for (uint32 i=0; i<2; i++) {
				for (uint32 j=0; j<2; j++) {
					status = mDevice->SetTunerSleep(static_cast<PT::Device::ISDB>(j), i, true);
					if (U::Error(L"Device::SetTunerSleep()", status)) return;
				}
			}

			for (uint32 i=0; i<2; i++) {
				for (uint32 j=0; j<2; j++) {
					status = mDevice->SetTunerSleep(static_cast<PT::Device::ISDB>(j), i, false);
					if (U::Error(L"Device::SetTunerSleep()", status)) return;
				}
			}
#endif
			mIsValid = true;
		}

		void Menu1()
		{
			bool done = false;
			while (done == false) {
				::wprintf(L"[foCXj[ (1^3y[W)]\n");
				::wprintf(L"0: (߂)\n");
				::wprintf(L"1: LNB dݒ\n");
				::wprintf(L"2: `lXL\n");
				::wprintf(L"3: `lݒ\n");
				::wprintf(L"4: TS-ID ݒ\n");
				::wprintf(L"5: Xe[^X\\n");
				::wprintf(L"6: Lv`%s\n", mStreamBusy ? L"~" : L"Jn");
				::wprintf(L"7:  2y[W\n");
				::wprintf(L"8: @\\n");

				#if 2 <= PUBLIC_LEVEL
				::wprintf(L"9:  3y[W\n");
				#endif
				::wprintf(L">");

				uint32	maxNumber =
				#if 3 <= PUBLIC_LEVEL
					9
				#else
					8
				#endif
				;

				uint32	number = U::GetNumber(maxNumber);
				if (number == 0) {
					done = true;
				} else {
					switch (number) {
						case 1: SetLnbPower(); break;
						case 2: ScanChannel(); break;
						case 3: SetChannel(); break;
						case 4: SetTsId(); break;
						case 5: mStatus->ErrorRateCounter(); break;
						case 6: Capture(); break;
						case 7: Menu2(); break;
						case 8: _ASSERT(mTest); mTest->CheckHardware(); break;

						#if 2 <= PUBLIC_LEVEL
						case 9: mDevice_->Menu3(); break;
						#endif
					}
				}
			}
		}

		void SetLnbPower()
		{
			status status;

			PT::Device::LnbPower lnbPower;
			status = mDevice->GetLnbPower(&lnbPower);
			if (U::Error(L"Device::GetLnbPower()", status)) return;

			const wchar_t *LABEL[] = { L"It", L"+15V", L"+11V" };

			::wprintf(L"[LNB dݒ]\n");
			::wprintf(L"0: (߂)\n");
			for (sint32 i=0; i<sizeof(LABEL)/sizeof(*LABEL); i++) {
				::wprintf(L"%d: %s%s\n", i+1, LABEL[i], (i==lnbPower) ? L" <ݒl>" : L"");
			}
			::wprintf(L">");
			uint32 number = U::GetNumber(3);
			if (number == 0) return;

			status = mDevice->SetLnbPower(static_cast<PT::Device::LnbPower>(number-1));
			if (U::Error(L"Device::SetLnbPower()", status)) return;
		}

		void ScanChannel()
		{
			PT::Device::ISDB	isdb;
			uint32				index;
			
			bool b = SelectIsdbIndex(&isdb, &index);
			if (b == false) return;

			switch (isdb) {
				case PT::Device::ISDB_S: ScanS(index, 0,  23); break;
				case PT::Device::ISDB_T: ScanT(index, 0, 112); break;
			}
		}

		void SetChannel()
		{
			PT::Device::ISDB	isdb;
			uint32				index;
			
			bool b = SelectIsdbIndex(&isdb, &index);
			if (b == false) return;

			uint32 max = (isdb == PT::Device::ISDB_S) ? 23 : 111;

			::wprintf(L"`lԍ͂ĂB(͈:0`%d)\n", max+1);
			::wprintf(L">");

			uint32 channel;
			::wscanf_s(L"%d", &channel);
			if (max < channel) {
				channel = max;
			}

			status status;
			if (isdb == PT::Device::ISDB_S) {
				status = ScanS(index, channel, channel);
			} else {
				status = ScanT(index, channel, channel);
			}
		}

		void SetTsId()
		{
			::wprintf(L"0: (߂)\n");
			::wprintf(L"1: S1\n");
			::wprintf(L"2: S2\n");
			::wprintf(L">");
			sint32	index = U::GetNumber(2);
			if (index == 0) return;

			::wprintf(L"TS-ID ͂ĂB(͈:0x0000`0xffff)\n");
			::wprintf(L">0x");

			uint32	id;
			::wscanf_s(L"%x", &id);
			if (0xffff < id) {
				id = 0xffff;
			}

			status	status = mDevice->SetIdS(index, id);
			if (U::Error(L"Device::SetIdS()", status)) return;
		}

		void Capture()
		{
			if (mStreamBusy) {
				for (uint32 i=0; i<PT::Device::ISDB_COUNT; i++) {
					for (uint32 j=0; j<2; j++) {
						_ASSERT(mStream[i*2+j]);
						mStream[i*2+j]->Stop();
					}
				}
				mStreamBusy = false;
				return;
			}

			for (uint32 i=0; i<PT::Device::ISDB_COUNT; i++) {
				for (uint32 j=0; j<2; j++) {
					if (mStream[i*2+j] == NULL) {
						mStream[i*2+j] = new EX::Stream(mDevice, static_cast<PT::Device::ISDB>(i), j);
					}
				}
			}

			mStream[0]->Run();
			mStream[1]->Run();
			mStream[2]->Run();
			mStream[3]->Run();

			mStreamBusy = true;
		}

		void Menu2()
		{
			::wprintf(L"[foCXj[ (2^3y[W)]\n");
			::wprintf(L"0: (߂)\n");
			::wprintf(L"1: nAvd\n");
			::wprintf(L"2: `[i[ȓd͐ݒ\n");
			::wprintf(L"3: `lݒ~4\n");
			//::wprintf(L"4: Mx\n");
			::wprintf(L"4: Gb^(FPGA 0x03)\n");
			//::wprintf(L"5: TS G[JE^Zbg\n");
			::wprintf(L">");

			uint32	number = U::GetNumber(4);
			if (number == 0) return;
			switch (number) {
				case 1: SetAmpPower(); break;
				case 2: SetTunerSleep(); break;
				case 3: ScanTest(); break;
				//case 4: ErrorRateMode(); break;
				case 4: Errata0x03(); break;
				//case 5: ResetTsErrorCounter(); break;
			}
		}

		void SetAmpPower()
		{
			::wprintf(L"0: (߂)\n");
			::wprintf(L"1: I\n");
			::wprintf(L"2: It\n");
			::wprintf(L">");

			uint32	number = U::GetNumber(2);
			if (number == 0) return;

			mDevice->SetAmpPowerT(number == 1);
		}

		void SetTunerSleep()
		{
			PT::Device::ISDB	isdb;
			uint32				index;
			
			bool	b = SelectIsdbIndex(&isdb, &index);
			if (b == false) return;

			bool	boolSleep;
			status status = mDevice->GetTunerSleep(isdb, index, &boolSleep);
			if (U::Error(L"Device::GetTunerSleep()", status)) return;

			const wchar_t	*LABEL[] = { L"", L"L" };

			::wprintf(L"0: (߂)\n");
			for (uint32 i=0; i<sizeof(LABEL)/sizeof(*LABEL); i++) {
				::wprintf(L"%d: %s%s\n", i+1, LABEL[i], (i==static_cast<uint32>(boolSleep)) ? L" (ݒl)" : L"");
			}
			::wprintf(L">");
			sint32	intSleep = U::GetNumber(2);
			if (intSleep == 0) return;

			boolSleep = (intSleep == 2) ? true : false;
			status = mDevice->SetTunerSleep(static_cast<PT::Device::ISDB>(isdb), index, boolSleep);
			if (U::Error(L"Device::SetTunerSleep()", status)) return;
		}

		status ScanS(uint32 tuner, uint32 startChannel, uint32 endChannel)
		{
			status	status = PT::STATUS_OK;

			::wprintf(L"                        :ύXw / N:NM / A:AbvN\n");
			const wchar_t	*STR =
			          L"---+----+---+-------+------+--+--+--+-------------------+----------------------\n";
			::wprintf(STR);
			::wprintf(L"No. Ch.  AGC clock carr  N A `[h/Slot   TS-ID (Hex)\n");
			::wprintf(L"        /128   (ppm)  (kHz)          1    2    3    4    1    2    3    4    5\n");
			::wprintf(STR);

			for (uint32 i=startChannel; i<=endChannel; i++) {
				status = ScanS(tuner, i);
				if (status) break;
			}

			::wprintf(STR);

			return status;
		}

		status ScanS(uint32 tuner, uint32 channel)
		{
			status	status = PT::STATUS_OK;

			bool	bs;
			uint32 number;
			GetChannelNameS(channel, &bs, &number);
			::wprintf(L"%3d %s%02d", channel, bs ? L"BS" : L"ND", number);
			
			status = mDevice->SetFrequency(PT::Device::ISDB_S, tuner, channel);
			if (U::Error(L"Device::SetFrequency()", status)) return status;

			uint32	startTime;
			OS::Time::Get(&startTime);

			PT::Device::TmccS	tmcc;
			::memset(&tmcc, 0, sizeof(tmcc));

			while (true) {
				uint32	currentTime;
				OS::Time::Get(&currentTime);

				status = mDevice->GetTmccS(tuner, &tmcc);
				if (status == PT::STATUS_OK) break;
				OS::Thread::Sleep(10);

				// SetFrequency()  1bȏオo߂Ă΁AMsƔf
				uint32	deltaTime = currentTime - startTime;
				if (1000 < currentTime-startTime) break; 
			}
			bool	tmccIsValid = (status == PT::STATUS_OK);

			uint32	cn = 0, currentAgc, maxAgc;

			status = mDevice->GetCnAgc(PT::Device::ISDB_S, tuner, &cn, &currentAgc, &maxAgc);
			if (U::Error(L"Device::GetCnAgc()", status)) return status;

			::wprintf(L" %3d", currentAgc);

			if (tmccIsValid == false) {
				::wprintf(L" (TMCC Ms)\n");
				return status;
			}

			PrintFrequencyOffset2(tuner, PT::Device::ISDB_S);

			::wprintf(L" %2d", tmcc.Indicator);
			::wprintf(L" %2d", tmcc.Emergency);
			::wprintf(L" %2d", tmcc.UpLink);

			for (uint32 i=0; i<4; i++) {
				uint32	mode = tmcc.Mode[i];
				if (mode != 0xf) {
					::wprintf(L" %1d/%2d", mode, tmcc.Slot[i]);
				} else {
					::wprintf(L" -/--");
				}
			}

			uint32	lastIndex = 0;
			for (uint32 i=0; i<8; i++) {
				if (tmcc.Id[i] != 0xffff) {
					lastIndex = i;
				}
			}

				for (uint32 i=0; i<8; i++) {
				uint32	id = tmcc.Id[i];
				if (id != 0xffff) {
					::wprintf(L" %04x", id);
				} else {
					if (i < lastIndex) {
						::wprintf(L" ----");
					}
				}
			}

			::wprintf(L"\n");

			return status;
		}

		status ScanT(uint32 tuner, uint32 startChannel, uint32 endChannel)
		{
			status	status = PT::STATUS_OK;

			::wprintf(L"---+---+---+-------+------+-+------------------------------------------\n");
			::wprintf(L"No. Ch. AGC clock carr S ϒ//C^[[u/ZOg \n");
			::wprintf(L"        255   (ppm)  (kHz)   AKw    BKw    CKw                   \n");
			::wprintf(L"---+---+---+-------+------+-+------------------------------------------\n");

			for (uint32 i=startChannel; i<=endChannel; i++) {
				status = ScanT(tuner, i);
				if (status) break;
			}

			::wprintf(L"----+---+---+-------+------+-+------------------------------------------\n");

			return status;
		}

		status ScanT(uint32 tuner, uint32 channel)
		{
			status	status = PT::STATUS_OK;

			bool	catv;
			uint32 number;
			GetChannelNameT(channel, &catv, &number);
			::wprintf(L"%3d %s%2d", channel, catv ? L"C" : L" ", number);

			status = mDevice->SetFrequency(PT::Device::ISDB_T, tuner, channel);
			if (U::Error(L"Device::SetFrequency", status)) return status;

			PT::Device::TmccT tmcc;
			::memset(&tmcc, 0, sizeof(tmcc));

			status = mDevice->GetTmccT(tuner, &tmcc);
			bool tmccIsValid = (status == PT::STATUS_OK);

			uint32	cn, currentAgc, maxAgc;
			status = mDevice->GetCnAgc(PT::Device::ISDB_T, tuner, &cn, &currentAgc, &maxAgc);
			if (U::Error(L"Device::GetCnAgc()", status)) return status;

			::wprintf(L" %3d", currentAgc);

			if (tmccIsValid) {
				PrintFrequencyOffset(tuner, PT::Device::ISDB_T);

				::wprintf(L" %d", tmcc.System);

				for (uint32 j=0; j<3; j++) {
					::wprintf(L" ");

					uint32 mode = tmcc.Mode[j];
					if (mode != 7) {
						::wprintf(L"%d", mode);
					} else {
						::wprintf(L"-");
					}

					::wprintf(L"/");

					uint32 rate = tmcc.Rate[j];
					if (mode != 7) {
						::wprintf(L"%d", rate);
					} else {
						::wprintf(L"-");
					}

					::wprintf(L"/");

					uint32 interleave = tmcc.Interleave[j];
					if (mode != 7) {
						::wprintf(L"%d", interleave);
					} else {
						::wprintf(L"-");
					}

					::wprintf(L"/");

					uint32 segment = tmcc.Segment[j];
					if (segment != 15) {
						::wprintf(L"%02d", segment);
					} else {
						::wprintf(L"--");
					}
				}
				::wprintf(L"\n");


			} else {
				::wprintf(L" (TMCC Ms)\n");
			}

			return status;
		}

		void PrintFrequencyOffset2(uint32 tuner, PT::Device::ISDB isdb)
		{
			sint32	offset[2];
			mDevice->GetFrequencyOffset(isdb, tuner, &offset[0], &offset[1]);

			bool	minus[2];
			
			for (uint32 k=0; k<2; k++) {
				minus[k] = (offset[k] < 0);
				if (minus[k]) {
					offset[k] *= -1;
				}
			}

			::wprintf(L" %s%3d.%02d", minus[0] ? L"-" : L"+", offset[0] / 100, offset[0] % 100);

			wchar_t		*si;
			if (isdb == PT::Device::ISDB_S) {
				offset[1] /= 1000;
				si = L"k";
			} else {
				si = L"";
			}
			::wprintf(L" %s%5d" , minus[1] ? L"-" : L"+", offset[1]);
		}

		void PrintFrequencyOffset(uint32 tuner, PT::Device::ISDB isdb)
		{
			sint32	offset[2];
			mDevice->GetFrequencyOffset(isdb, tuner, &offset[0], &offset[1]);

			bool	minus[2];
			
			for (uint32 k=0; k<2; k++) {
				minus[k] = (offset[k] < 0);
				if (minus[k]) {
					offset[k] *= -1;
				}
			}

			::wprintf(L" %s%3d.%02d", minus[0] ? L"-" : L"+", offset[0] / 100, offset[0] % 100);

			wchar_t		*si;
			offset[1] /= 1000;
			si = L"k";
			::wprintf(L" %s%5d" , minus[1] ? L"-" : L"+", offset[1]/*, si*/);
		}

		void ScanTest()
		{
			ScanS(0,  7,  7);
			ScanS(1, 15, 15);

			ScanT(0, 70, 70);
			ScanT(1, 71, 71);

			for (uint32 i=0; i<2; i++) {
				for (uint32 j=0; j<PT::Device::ISDB_COUNT; j++) {
					mDevice->ResetCorrectedErrorCount(static_cast<PT::Device::ISDB>(j), i);
				}
			}
		}

		void PrintErrorRate2(PT::Device::ISDB isdb_)
		{
			for (uint32 i=0; i<2; i++) {
				PT::Device::LayerIndex	layerIndex = (isdb_ == 	PT::Device::ISDB_S)
					? PT::Device::LAYER_INDEX_H : PT::Device::LAYER_INDEX_B;
				PT::Device::ErrorRate	errorRate;
				mDevice->GetCorrectedErrorRate(isdb_, i, layerIndex, &errorRate);

				wchar_t		isdb = (isdb_ == PT::Device::ISDB_S) ? L'S' : L'T';
				wchar_t		layer = (isdb_ == PT::Device::ISDB_S) ? ((layerIndex == 0) ? L'' : L'') : static_cast<wchar_t>(L'A'+layerIndex);
				const wchar_t *space = (isdb_ == PT::Device::ISDB_S) ? L"" : L" ";
				::wprintf(L" `[iԍ:%d ISDB-%c %c%sKw ", i, isdb, layer, space);
				::wprintf(L"\n");
			}
		}

		void SetChannel22(PT::Device::ISDB isdb, uint32 channel)
		{
			if (isdb == PT::Device::ISDB_S) {
				ScanS(0, channel, channel);
				ScanS(1, channel, channel);
			} else {
				ScanT(0, channel, channel);
				ScanT(1, channel, channel);
			}
		}

		void ErrorRateMode()
		{
			::wprintf(L"0: (߂)\n");
			::wprintf(L"1: q\n");
			::wprintf(L"2: n\n");
			::wprintf(L">");

			uint32	number = U::GetNumber(2);
			if (number == 0) return;

			PT::Device::ISDB	isdb = static_cast<PT::Device::ISDB>(number-1);

			uint32	channel = 0;

			SetChannel22(isdb, channel);

			while (true) {
				::wprintf(L"0: (߂)\n");
				::wprintf(L"1: `l--\n");
				::wprintf(L"2: `l++\n");
				::wprintf(L"3: G[[g\n");
				::wprintf(L">");

				uint32	number = U::GetNumber(3);
				if (number == 0) return;

				switch (number) {
					case 1:
					{
						if (channel) channel--;
						SetChannel22(isdb, channel);
						break;
					}
					case 2:
					{
						channel++;
						SetChannel22(isdb, channel);
						break;
					}
					case 3:
					{
						//PrintErrorRate2(isdb);
						break;
					}
				}
			}
		}

		void Errata0x03()
		{
			_ASSERT(mErrata);
			mErrata->Check();
		}

		void ResetTsErrorCounter()
		{
			PT::Device::ISDB	isdb;
			uint32				index;

			bool	b = SelectIsdbIndex(&isdb, &index);
			if (b == false) return;

			bool	enabled;
			status	status = mDevice->GetTransferEnabled(isdb, index, &enabled);

			if (enabled) {
				::wprintf(L"DMA 쒆̓JE^[̃Zbg͂ł܂B\n");
			}
			//mDevice->SetTransferTestMode(isdb, index, false, 0, false, true);
		}

		void GetChannelNameS(uint32 channel, bool *bs, uint32 *number) const
		{
			_ASSERT(bs);
			_ASSERT(number);

			if (channel < 12) {
				*bs = true;
				*number = 1 + 2*channel;
			} else if (channel < 24) {
				channel -= 12;

				*bs = false;
				*number = 2 + 2*channel;
			} else {
				channel -= 24;

				*bs = false;
				*number = 1 + 2*channel;
			}
		}

		void GetChannelNameT(uint32 channel, bool *catv, uint32 *number) const
		{
			static const uint8	TABLE[][3] = {
				{   2,  0,   3 },
				{  12,  1,  22 },
				{  21,  0,  12 },
				{  62,  1,  63 },
				{ 112,  0,  62 }
			};

			for (uint32 i=0; i<sizeof(TABLE)/sizeof(*TABLE); i++) {
				if (channel <= TABLE[i][0]) {
					_ASSERT(catv); *catv = TABLE[i][1] ? true : false;
					_ASSERT(number); *number = channel + TABLE[i][2] - TABLE[i][0];
					break;
				}
			}
		}

		bool SelectIsdbIndex(PT::Device::ISDB *isdb, uint32 *index)
		{
			::wprintf(L"0: (߂)\n");
			::wprintf(L"1: S1\n");
			::wprintf(L"2: S2\n");
			::wprintf(L"3: T1\n");
			::wprintf(L"4: T2\n");
			::wprintf(L">");

			sint32	select = U::GetNumber(4);
			if (select == 0) return false;

			select--;

			_ASSERT(isdb);
			*isdb = static_cast<PT::Device::ISDB>(select / 2);

			_ASSERT(index);
			*index = select % 2;

			return true;
		}

	};
}
}

#endif
