#ifndef _EX_Status_H
#define _EX_Status_H

#include "Prefix.h"
#include "EARTH_PT3.h"

namespace EARTH {
namespace EX {
	class Status {
	public:
		Status(PT::Device *device) : mDevice(device)
		{
		}

		~Status()
		{
		}

		void ErrorRateCounter()
		{
			Print_New();
		}

	protected:
		PT::Device	*const mDevice;

		void PrintErrorRate(const PT::Device::ErrorRate *errorRate)
		{
			_ASSERT(errorRate);
			uint32 numerator   = errorRate->Numerator;
			uint32 denominator = errorRate->Denominator;

			if ((numerator == 0) || (denominator == 0)) {
				::wprintf(L" %7d", 0);
				return;
			}

			uint32	d = 0;
			while (numerator <= denominator) {
				numerator *= 10;
				d++;
			}

			double	f = static_cast<double>(numerator) / denominator;

			::wprintf(L" %.2fe-%d", f, d);
		}

		void PrintLabel(PT::Device::ISDB isdb, uint32 index)
		{
			wchar_t c = (isdb == PT::Device::ISDB_S) ? L'S' : L'T';
			::wprintf(L"%c%d", c, index+1);
		}

		void PrintCnAGC(PT::Device::ISDB isdb, uint32 index)
		{
			uint32 cn, currentAgc, maxAgc;
			status status = mDevice->GetCnAgc(isdb, index, &cn, &currentAgc, &maxAgc);
			if (U::Error(L"Device::GetCnAgc()", status)) return;

			::wprintf(L" %2d.%02d %3d", cn / 100, cn % 100, currentAgc);
		}

		void PrintLevel(PT::Device::ISDB isdb, uint32 index)
		{
			if (isdb == PT::Device::ISDB_S) {
				::wprintf(L"         ");
			} else {
				float	level;
				status	status = mDevice->GetRFLevel(index, &level);
				if (U::Error(L"Device::GetRFLevel()", status)) return;
				::wprintf(L" %8.3f", level);
			}
		}

		void PrintErrorPacket(PT::Device::ISDB isdb, uint32 index)
		{
			uint32 count;
			mDevice->GetErrorCount(isdb, index, &count);

			::wprintf(L" %16d", BIT_SHIFT_MASK(count, 0, 24));
		}

		void PrintErrorRateRS(PT::Device::ISDB isdb, uint32 index)
		{
			uint32 layerCount = (isdb == PT::Device::ISDB_S) ? PT::Device::LAYER_COUNT_S : PT::Device::LAYER_COUNT_T;
			for (uint32 i=0; i<layerCount; i++) {
				PT::Device::ErrorRate errorRate;
				mDevice->GetCorrectedErrorRate(
							isdb, index, static_cast<PT::Device::LayerIndex>(i), &errorRate);

				PrintErrorRate(&errorRate);
			}

			if (isdb == PT::Device::ISDB_S) {
					::wprintf(L"        ");
			}
		}

		void PrintErrorRateVi(PT::Device::ISDB isdb, uint32 index)
		{
			PT::Device::ErrorRate errorRate;
			mDevice->GetInnerErrorRate(isdb, index, &errorRate);

			PrintErrorRate(&errorRate);
		}

		void Print_New()
		{
			::wprintf(L"--+-----+---+--------+----------------+-------+-------+-------+-------\n");
			::wprintf(L"     AGC RFx G[pPbg [ꂽrbg[g   ]\n");
			::wprintf(L"   C/N          (dBm)                  [[h\       ] r^r\n");
			::wprintf(L"    (dB)                               Kw  Kw\n");
			::wprintf(L"                                        AKw   BKw   CKw\n");
			::wprintf(L"--+-----+---+--------+----------------+-------+-------+-------+-------\n");

			for (uint32 i=0; i<2; i++) {
				PT::Device::ISDB isdb = static_cast<PT::Device::ISDB>(i);
				for (uint32 index=0; index<2; index++) {
					PrintLabel      (isdb, index);
					PrintCnAGC      (isdb, index);
					PrintLevel      (isdb, index);
					PrintErrorPacket(isdb, index);
					PrintErrorRateRS(isdb, index);
					PrintErrorRateVi(isdb, index);
					::wprintf(L"\n");
				}
			}
		}
	};
}
}

#endif
