#ifndef _EX_Stream_H
#define _EX_Stream_H

#include "Prefix.h"
#include "EARTH_PT3.h"
#include "EX_Stream.h"
#include "OS_Memory.h"
#include "OS_Thread.h"
#include "OS_File.h"
#include "EX_Buffer.h"

namespace EARTH {
namespace EX {
	class Stream {
	public:
		Stream(PT::Device *device, PT::Device::ISDB isdb, uint32 tunerIndex)
			: mDevice(device), mISDB(isdb), mTunerIndex(tunerIndex)
		{
			mFile = NULL;
			mBuffer = NULL;
			mThread = NULL;

			const wchar_t	CHAR[PT::Device::ISDB_COUNT] = { L'S', L'T' };

			_ASSERT(mISDB < sizeof(CHAR)/sizeof(*CHAR));
			::swprintf_s(mString, sizeof(mString)/sizeof(*mString), L"ISDB-%c%d", CHAR[mISDB], mTunerIndex+1);
		}

		~Stream()
		{
			Stop();

			delete_(mBuffer);
		}

		void Run()
		{
			mQuit = false;

			_ASSERT(mThread == NULL);
			mThread = new OS::Thread(Thread_, this);
			_ASSERT(mThread);
		}

		void Stop()
		{
			if (mThread) {
				mQuit = true;
				delete_(mThread);
			}
		}

	protected:
		PT::Device				*const mDevice;
		const PT::Device::ISDB	mISDB;
		const uint32			mTunerIndex;
		wchar_t					mString[8];

		OS::File				*mFile;
		Buffer					*mBuffer;
		OS::Thread				*mThread;
		
		volatile bool			mQuit;
		uint32					mBlockIndex;

		bool					mWriteMessage;
		bool					mBadSyncByteMessage;

		enum {
			NOT_SYNC_BYTE = 0x74	// 0x47 łȂCӂ̒l
		};

		// [TS obt@̍\]
		// 1. OS::File NX FILE_FLAG_NO_BUFFERING w肵Ă邽߁AWriteFile() ŏރTCY̓ZN^TCY̔{łKvB
		// 2. ł̓ZN^TCY 4096 Ɖ肷Ώ\łB
		// 3. TS pPbgTCY 188 oCgłApPbgfȂ悤ɏ݃TCY TS pPbgTCY̔{ł邱Ƃ]܂B
		// 4. ď݃TCY 4096  188 ̍ŏ{ł 4096*47(jbgTCY) ̔{ɂׂłB
		// 5. ł̓ANZXlāA݃TCY 8 jbgTCYƂB
		// 6. ܂AOobt@Ŝ̃TCY͏݃TCY 32 {ƂB

		enum {
			BLOCK_SIZE  = OS::Memory::PAGE_SIZE * 47 * 8,	// WriteFile() ŏރTCY
			BLOCK_COUNT = 32,								// Oobt@ BLOCK_SIZE  32 {
			BUFFER_SIZE = BLOCK_SIZE * BLOCK_COUNT			// Oobt@̃TCY
		};

		static void Thread_(void *argument)
		{
			Stream	 *that = static_cast<Stream *>(argument);
			that->Thread();
		}

		status Thread()
		{
			status	status;
			
			status = Begin();
			if (status) return status;

			status = Loop();
			if (status) return status;

			status = End();
			if (status) return status;

			return status;
		}

		status Begin()
		{
			status	status;

			mBlockIndex = 0;

			wchar_t		fileName[MAX_PATH];
			::swprintf_s(fileName, MAX_PATH, L"%s.ts", mString);

			if (mFile == NULL) {
				mFile = new OS::File;
			}
			status = mFile->Open(fileName);
			if (status) return status;

			if (mBuffer == NULL) {
				mBuffer = new Buffer(mDevice);

				status = mBuffer->Alloc(BLOCK_SIZE, BLOCK_COUNT);
				if (status) return status;
			}

			_ASSERT(mBuffer);
			uint8	*ptr = static_cast<uint8 *>(mBuffer->Ptr(0));
			_ASSERT(ptr);

			for (uint32 i=0; i<BLOCK_COUNT; i++) {
				ptr[BLOCK_SIZE*i] = NOT_SYNC_BYTE;	// oCg NOT_SYNC_BYTE 
				status = mBuffer->SyncCpu(i);		// CPU LbVtbV
				if (U::Error(L"EX_Buffer::SyncCpu()", status)) return status;
			}
			
			mWriteMessage       = false;
			mBadSyncByteMessage = false;

			status = mDevice->SetTransferTestMode(mISDB, mTunerIndex);
			if (U::Error(L"Device::SetTransferTestMode()", status)) return status;

			_ASSERT(mBuffer);
			uint64	pageAddress = mBuffer->PageDescriptorAddress();

			status = mDevice->SetTransferPageDescriptorAddress(mISDB, mTunerIndex, pageAddress);
			if (U::Error(L"Device::SetTransferAddress()", status)) return status;

			status = mDevice->SetTransferEnabled(mISDB, mTunerIndex, true);
			if (U::Error(L"Device::SetTransferEnabled(true)", status)) return status;

			return status;
		}

		status Loop()
		{
			_ASSERT(mBuffer);
			uint8	*ptr = static_cast<uint8 *>(mBuffer->Ptr(0));
			_ASSERT(ptr);

			while (true) {
				OS::Thread::Sleep(1);

				if (mQuit) break;

				uint32	nextWriteIndex = mBlockIndex + 1;
				if (BLOCK_COUNT <= nextWriteIndex) {
					nextWriteIndex = 0;
				}

				bool	b = CheckReady(nextWriteIndex);
				if (b) {
					if (mWriteMessage == false) {
						mWriteMessage = true;
						::wprintf(L"%s.ts ̏oJn܂B\n", mString);
					}

					Write(mBlockIndex);

					mBlockIndex++;
					if (BLOCK_COUNT <= mBlockIndex) {
						mBlockIndex = 0;
					}
				}
			}

			return 0;
		}

		status End()
		{
			status	status;

			PT::Device::TransferInfo	info;
			status = mDevice->GetTransferInfo(mISDB, mTunerIndex, &info);
			if (U::Error(L"Device::GetTransferInfo()", status)) return status;

			::wprintf(L"%s Xe[^X:", mString);
			if ( info.InternalFIFO_A_Overflow ) ::wprintf(L" [Internal FIFO A Overflow]");
			if ( info.InternalFIFO_A_Underflow) ::wprintf(L" [Internal FIFO A Underflow]");
			if ( info.ExternalFIFO_Overflow   ) ::wprintf(L" [External FIFO Overflow]");
			if (!info.ExternalFIFO_Overflow   ) ::wprintf(L" [External FIFO őgp:%d oCg]", info.ExternalFIFO_MaxUsedBytes);
			if ( info.InternalFIFO_B_Overflow ) ::wprintf(L" [Internal FIFO B Overflow]");
			if ( info.InternalFIFO_B_Underflow) ::wprintf(L" [Internal FIFO B Underflow]");
			::wprintf(L"\n");

			status = mDevice->SetTransferEnabled(mISDB, mTunerIndex, false);
			if (U::Error(L"Device::SetTransferEnabled(*, *, false)", status)) return status;
			
			if (mFile) {
				mFile->Close();
				delete_(mFile);
			}

			return status;
		}

		bool CheckReady(uint32 blockIndex)
		{
			_ASSERT(mBuffer);
			volatile uint8	*ptr  = static_cast<uint8 *>(mBuffer->Ptr(blockIndex));
			_ASSERT(ptr);

			uint8	data = ptr[0];
			status	status = mBuffer->SyncCpu(blockIndex);	// LbVCŜɓnĐf[^ǂݍ񂾂Ƃ͌Ȃ߁ACPU LbVtbV
			if (U::Error(L"EX_Buffer::SyncCpu()", status)) return false;

			if (data == 0x47) return true;
			if (data == NOT_SYNC_BYTE) return false;

			if (mBadSyncByteMessage == false) {
				mBadSyncByteMessage = true;
				::wprintf(L"%s ̓oCg 0x47 ł NOT_SYNC_BYTE ł܂B\n", mString);
			}

			return false;
		}

		status Write(uint32 blockIndex)
		{
			status	status;

			_ASSERT(mBuffer);
			status = mBuffer->SyncIo(blockIndex);	// DMA ̈ǂݏoOɓ
			if (U::Error(L"EX_Buffer::SyncIo()", status)) return status;

			uint8	*ptr  = static_cast<uint8 *>(mBuffer->Ptr(blockIndex));
			_ASSERT(ptr);

			mFile->Write(ptr, BLOCK_SIZE);

			ptr[0] = NOT_SYNC_BYTE;			// oCg NOT_SYNC_BYTE Ă
			status = mBuffer->SyncCpu(blockIndex);	// CPU LbVtbV
			if (U::Error(L"EX_Buffer::SyncCpu()", status)) return status;
		}
	};
}
}

#endif
