#ifndef _OS_Thread_H
#define _OS_Thread_H

#include "Prefix.h"
#include <Windows.h>

namespace EARTH {
namespace OS {
	class Thread {
	public:
		typedef void (*Function)(void *);

		Thread(Function function, void *argument) : mFunction(function), mArgument(argument)
		{
			mThread = NULL;

			_ASSERT(mThread == NULL);
			mThread = ::CreateThread(NULL, 0, ThreadFunction_, this, 0, NULL);
			if (mThread == NULL) { _ASSERT(false); return; }
		}

		~Thread()
		{
			if (mThread) {
				DWORD	result = ::WaitForSingleObject(mThread, INFINITE);
				_ASSERT(result == WAIT_OBJECT_0);
				::CloseHandle(mThread);
				mThread = NULL;
			}
		}

		bool IsValid() const
		{
			return (mThread != NULL);
		}

		static void Sleep(uint32 ms)
		{
			::Sleep(ms);
		}

	protected:
		const Function	mFunction;
		void			*const mArgument;
		HANDLE			mThread;

		void ThreadFunction()
		{
			_ASSERT(mFunction);
			mFunction(mArgument);
		}

		static uint32 __stdcall ThreadFunction_(void *argument)
		{
			Thread	*that = static_cast<Thread *>(argument);
			_ASSERT(that);

			that->ThreadFunction();

			return 0;
		}
	};
}
}

#endif
